AuthorisationContext = (function () {
    AuthorisationContext = function (config) {
        this.roles = null;
        this.privileges = null;
        this.promise = null;
        this.data = null;
        this.config = config;

        this.context = new AuthenticationContext(config);
        this.context.handleWindowCallback();
        this.getData();
    };

    AuthorisationContext.prototype.context = function () {
        this.context;
    }
    AuthorisationContext.prototype.login = function () {
        this.context.login();
    }
    AuthorisationContext.prototype.logout = function () {
        this.context.logOut();
    }
    AuthorisationContext.prototype.handleCallback = function () {
        this.context.handleCallback();
    }
    AuthorisationContext.prototype.login = function () {
        this.context.login();
    }

    AuthorisationContext.prototype.hasRole = function (role) {
        return this.getRoles().then(roles => roles.some(r => r === role));
    }
    AuthorisationContext.prototype.hasPrivilege = function (privilege) {
        return this.getPrivileges().then(privileges => privileges.some(p => p === privilege));
    }
    AuthorisationContext.prototype.getRoles = function () {
        return this.getData().then(data => data.map(r => r.RoleTag));
    }
    AuthorisationContext.prototype.getPrivileges = function () {
        return this.getData().then(data => {
            return data.reduce((acc, value) => { return acc.concat(value.Privileges.map(p => p.PrivilegeTag)) }, []);
        });
    }

    AuthorisationContext.prototype.userInfo = function () {
        if (this.context) {
            return this.context.getCachedUser();
        }
        return {};
    }
    AuthorisationContext.prototype.isAuthenticated = function () {
        return this.userInfo() && this.context.getCachedToken(this.config.clientId);
    }
    AuthorisationContext.prototype.getData = function () {
        if (!this.isAuthenticated()) {
            this.context.login();
        }

        if (this.data != null) {
            return Promise.resolve(this.data);
        }
        if (this.promise != null) {
            return this.promise;
        }

        let that = this;

        this.promise = new Promise(function (resolve, reject) {

            that.context.acquireToken(that.config.getRoleResource, (err, token) => {

                fetch(that.config.getRoleUrl, {
                    headers: {
                        'Ocp-Apim-Subscription-Key': that.config.ocpApimSubscriptionKey,
                        'OrganizationId': that.config.organizationId,
                        'Accept': '*/*',
                        'Authorization': 'Bearer ' + token
                    },
                    method: 'GET',
                    mode: 'cors',
                })
                    .then(response => response.json())
                    .then(data => {
                        resolve(data.Result);
                    });
            });
        });
        return this.promise;
    }

    return AuthorisationContext;
}());
